import {
  CreditGracePeriod,
  CreditInformation,
  CreditInstallment,
  Event,
  Operation,
  RescheduleReason,
  Schedule,
  ScheduleCancellationReason,
  ScheduleContext,
  ScheduleStatus,
  VisitAddress
} from '@agiliza/api/domain'
import {
  AgendamentoDadosCreditoInteresseApiModel,
  AgendamentoVisitaClienteApiModel,
  AgendamentoVisitaOperacaoApiModel,
  AgendamentoVisitaSituacaoApiModel,
  AgendamentoVisitaSituacaoCompletoApiModel,
  ContextoVisitaResponseApiModel,
  CreditoCarenciaApiModel,
  EnderecoApiModel,
  EventoVisitaApiModel,
  MotivoCancelamentoApiModel,
  MotivoReagendamentoApiModel
} from '@microcredito/client'

import { ProjectDocumentMapper } from './project'
import { DomainAdapter } from './shared'

export class CreditInformationAdapter implements DomainAdapter<AgendamentoDadosCreditoInteresseApiModel, CreditInformation> {
  public mapApiModelToDomain = (apimodel: AgendamentoDadosCreditoInteresseApiModel): CreditInformation => ({
    creditValue: apimodel.valor,
    creditInstallment: apimodel.numeroParcelas,
    creditObjective: apimodel.objetivo,
    creditGracePeriod: apimodel.carencia,
  })
}

export class VisitAddressAdapter implements DomainAdapter<EnderecoApiModel, VisitAddress> {
  public mapApiModelToDomain = (apimodel: EnderecoApiModel): VisitAddress => ({
    id: apimodel.id || '',
    cep: apimodel.cep,
    city: apimodel.municipio,
    state: apimodel.estado,
    complement: apimodel.complemento || '',
    district: apimodel.bairro,
    number: apimodel.numero,
    street: apimodel.logradouro,
  })
}

type Agendamento = Omit<AgendamentoVisitaClienteApiModel, 'creditoInteresse' | 'enderecoVisita'>

export class ScheduleAdapter implements DomainAdapter<Agendamento, Schedule> {
  // private userClientMapper: UserClientMapper
  // private addressAdapter: VisitAddressAdapter
  // private creditInformationAdapter: CreditInformationAdapter
  private statusAdapter: ScheduleStatusAdapter
  private documentAdapter: ProjectDocumentMapper
  constructor() {
    // this.userClientMapper = new UserClientMapper()
    // this.addressAdapter = new VisitAddressAdapter()
    // this.creditInformationAdapter = new CreditInformationAdapter()
    this.statusAdapter = new ScheduleStatusAdapter()
    this.documentAdapter = new ProjectDocumentMapper()
  }

  public mapApiModelToDomain = (agendamento: Agendamento): Schedule => {
    const result: Schedule = {
      id: agendamento.id || '',
      number: agendamento.numero,
      date: agendamento.data,
      events: agendamento.eventos,
      documents: agendamento.documentos.map(this.documentAdapter.mapApiModelToDomain),
      status: this.statusAdapter.mapApiModelToDomain(agendamento.situacao),
      // user: this.userClientMapper.mapApiModelToDomain(agendamento.cliente),
      // complement: agendamento.enderecoVisita
      //   ? {
      //       address: this.addressAdapter.mapApiModelToDomain(agendamento.enderecoVisita),
      //       interestCredit: agendamento.creditoInteresse && this.creditInformationAdapter.mapApiModelToDomain(agendamento.creditoInteresse),
      //     }
      //   : ({} as any),
      // details: agendamento.solicitacaoReagendamento
      //   ? {
      //       rescheduleDate: agendamento.solicitacaoReagendamento.dataReagendamento,
      //       reason: agendamento.solicitacaoReagendamento.codigoMotivo,
      //       justification: agendamento.solicitacaoReagendamento.justificativa,
      //     }
      //   : agendamento.solicitacaoCancelamento
      //   ? {
      //       reason: agendamento.solicitacaoCancelamento.codigoMotivo,
      //       justification: agendamento.solicitacaoCancelamento.justificativa,
      //     }
      //   : ({} as any),
    }

    return result
  }
}

export class ScheduleStatusAdapter implements DomainAdapter<AgendamentoVisitaSituacaoApiModel, ScheduleStatus> {
  private scheduleOperationAdapter: ScheduleOperationAdapter
  constructor() {
    this.scheduleOperationAdapter = new ScheduleOperationAdapter()
  }

  public mapApiModelToDomain = (apimodel: AgendamentoVisitaSituacaoApiModel): ScheduleStatus => {
    const situacao = apimodel as AgendamentoVisitaSituacaoCompletoApiModel
    const result: ScheduleStatus = {
      id: situacao.codigo || '',
      description: situacao.descricao,
      operations: situacao.operacoes?.filter((op) => op.permissao !== 'cliente').map(this.scheduleOperationAdapter.mapApiModelToDomain) ?? [],
    }
    return result
  }
}

export class ScheduleOperationAdapter implements DomainAdapter<AgendamentoVisitaOperacaoApiModel, Operation> {
  public mapApiModelToDomain = (operacao: AgendamentoVisitaOperacaoApiModel): Operation => ({
    id: operacao.codigo,
    description: operacao.descricao,
    descriptionTransition: operacao.descricaoTransicao,
    permission: operacao.permissao === 'cliente' ? 'customer' : 'broker',
  })
}

export class RescheduleReasonAdapter implements DomainAdapter<MotivoReagendamentoApiModel, RescheduleReason> {
  public mapApiModelToDomain = (apimodel: MotivoReagendamentoApiModel): RescheduleReason => ({
    id: apimodel.codigo,
    description: apimodel.descricao,
  })
}

export class ScheduleCancellationReasonAdapter implements DomainAdapter<MotivoCancelamentoApiModel, ScheduleCancellationReason> {
  public mapApiModelToDomain = (apimodel: MotivoCancelamentoApiModel): ScheduleCancellationReason => ({
    id: apimodel.codigo,
    description: apimodel.descricao,
  })
}

export class VisitEventAdapter implements DomainAdapter<EventoVisitaApiModel, Event> {
  public mapApiModelToDomain = (apimodel: EventoVisitaApiModel): Event => ({
    id: apimodel.id || '',
    title: apimodel.titulo,
    date: apimodel.data,
    description: apimodel.descricao,
  })
}

// Parcelas
export class CreditInstallmentAdapter implements DomainAdapter<number, CreditInstallment> {
  public mapApiModelToDomain = (apimodel: number) => {
    const result: CreditInstallment = {
      id: apimodel.toString(),
      description: apimodel.toString(),
    }
    return result
  }
}

// Carência
export class CreditGracePeriodAdapter implements DomainAdapter<CreditoCarenciaApiModel, CreditGracePeriod> {
  public mapApiModelToDomain = (apimodel: CreditoCarenciaApiModel) => {
    const result: CreditGracePeriod = {
      id: apimodel.codigo,
      description: apimodel.descricao,
    }
    return result
  }
}

export class ScheduleContextAdapter implements DomainAdapter<ContextoVisitaResponseApiModel, ScheduleContext> {
  // private objectiveAdapter: CreditObjectiveAdapter
  private gracePeriodAdapter: CreditGracePeriodAdapter
  private installmentAdapter: CreditInstallmentAdapter
  // private dueDateAdapter: CreditDueDateAdapter
  // private productAdapter: CreditProductAdapter
  constructor() {
    // this.objectiveAdapter = new CreditObjectiveAdapter()
    this.gracePeriodAdapter = new CreditGracePeriodAdapter()
    this.installmentAdapter = new CreditInstallmentAdapter()
    // this.dueDateAdapter = new CreditDueDateAdapter()
    // this.productAdapter = new CreditProductAdapter()
  }
  public mapApiModelToDomain = (apimodel: ContextoVisitaResponseApiModel) => {
    const result: ScheduleContext = {
      gracePeriods: apimodel.carencias.map(this.gracePeriodAdapter.mapApiModelToDomain),
      installments: apimodel.parcelas.map(this.installmentAdapter.mapApiModelToDomain),
    }
    return result
  }
}
