import { connect } from 'react-redux'
import { bindActionCreators, Dispatch } from 'redux'

import { BusinessData, InvolvedPerson } from '@agiliza/api/domain'
import { StoreState } from '@agiliza/redux'
import * as entBusinessData from '@agiliza/redux/entities/proposalData/businessData'
import * as entCustomerData from '@agiliza/redux/entities/proposalData/customerData'
import * as entProposal from '@agiliza/redux/entities/proposalData/proposal'
import * as ucAuthentication from '@agiliza/redux/useCases/authentication'
import * as ucProposalDataBD from '@agiliza/redux/useCases/proposalData/businessData'
import * as ucProposalDataContext from '@agiliza/redux/useCases/proposalData/context'
import * as ucProposalDataCD from '@agiliza/redux/useCases/proposalData/customerData'
import * as ucSimulation from '@agiliza/redux/useCases/simulation'
import { createSelector } from '@reduxjs/toolkit'

export interface ConnectedProps {
  fetching: boolean
  proposalId: string
  personId: string
  reset: () => void
  customerData: Omit<InvolvedPerson, 'id'>
  businessData: BusinessData
  fetchContext: typeof ucProposalDataContext.actions.fetchContext
  getContextAddress: typeof ucAuthentication.actions.getContext
  addIndentification: typeof ucProposalDataCD.actions.addIndentification
  updateCDIndentification: typeof ucProposalDataCD.actions.updateIdentification
  updatePersonalData: typeof ucProposalDataCD.actions.updatePersonalData
  updateCDAddress: typeof ucProposalDataCD.actions.updateAddress
  updateSourceIncome: typeof ucProposalDataCD.actions.updateSourceIncome
  updateSpouseData: typeof ucProposalDataCD.actions.updateSpouseData
  updateHome: typeof ucProposalDataCD.actions.updateHome
  updateBDIdentification: typeof ucProposalDataBD.actions.updateIdentification
  updateBDAddress: typeof ucProposalDataBD.actions.updateAddress
  updateStockSalesCosts: typeof ucProposalDataBD.actions.updateStockSalesCosts
  sendProposal: typeof ucSimulation.actions.sendProposal
  getCDIdentification: typeof ucProposalDataCD.actions.getIdentification
  getCustomerData: typeof ucProposalDataCD.actions.getCustomerData
  getBusinessData: typeof ucProposalDataBD.actions.getBusinessData
}

type StateProps = Pick<ConnectedProps, 'fetching' | 'proposalId' | 'personId' | 'businessData' | 'customerData'>
type DispatchProps = Pick<
  ConnectedProps,
  | 'reset'
  | 'addIndentification'
  | 'updateCDIndentification'
  | 'fetchContext'
  | 'updatePersonalData'
  | 'updateCDAddress'
  | 'getContextAddress'
  | 'updateSourceIncome'
  | 'updateSpouseData'
  | 'updateHome'
  | 'updateBDIdentification'
  | 'updateBDAddress'
  | 'updateStockSalesCosts'
  | 'sendProposal'
  | 'getCDIdentification'
  | 'getCustomerData'
  | 'getBusinessData'
>

const isFetching = createSelector(
  (state: StoreState) => ucProposalDataContext.selectors.isFetching(state.ui.login),
  (state: StoreState) => ucAuthentication.selectors.isFetching(state.useCases.authentication),
  (state: StoreState) => ucProposalDataCD.selectors.isFetching(state.useCases.proposalData.customerData),
  (state: StoreState) => ucProposalDataCD.selectors.isGettingCustomerData(state.useCases.proposalData.customerData),
  (state: StoreState) => ucProposalDataBD.selectors.isFetching(state.useCases.proposalData.businessData),
  (state: StoreState) => ucProposalDataBD.selectors.isGettingBusinessData(state.useCases.proposalData.businessData),
  (state: StoreState) => ucSimulation.selectors.isSendingProposal(state.useCases.simulation),
  (...conditions) => conditions.some((c) => c)
)

const mapStateToProps = (state: StoreState): StateProps => ({
  fetching: isFetching(state),
  proposalId: entProposal.selectors.getProposalId(state.entities.proposalData.proposal),
  personId: entProposal.selectors.getPersonlId(state.entities.proposalData.proposal),
  customerData: entCustomerData.selectors.getCustomerData(state.entities.proposalData.customerData),
  businessData: entBusinessData.selectors.getBusinessData(state.entities.proposalData.businessData),
})

const mapDispatchToProps = (dispatch: Dispatch): DispatchProps =>
  bindActionCreators(
    {
      reset: () => ({ type: 'RESET_STORE' }),
      addIndentification: ucProposalDataCD.actions.addIndentification,
      updateCDIndentification: ucProposalDataCD.actions.updateIdentification,
      fetchContext: ucProposalDataContext.actions.fetchContext,
      updatePersonalData: ucProposalDataCD.actions.updatePersonalData,
      updateCDAddress: ucProposalDataCD.actions.updateAddress,
      getContextAddress: ucAuthentication.actions.getContext,
      updateSourceIncome: ucProposalDataCD.actions.updateSourceIncome,
      updateSpouseData: ucProposalDataCD.actions.updateSpouseData,
      updateHome: ucProposalDataCD.actions.updateHome,
      updateBDAddress: ucProposalDataBD.actions.updateAddress,
      updateBDIdentification: ucProposalDataBD.actions.updateIdentification,
      updateStockSalesCosts: ucProposalDataBD.actions.updateStockSalesCosts,
      sendProposal: ucSimulation.actions.sendProposal,
      getBusinessData: ucProposalDataBD.actions.getBusinessData,
      getCustomerData: ucProposalDataCD.actions.getCustomerData,
      getCDIdentification: ucProposalDataCD.actions.getIdentification,
    },
    dispatch
  )

export const connected = connect(mapStateToProps, mapDispatchToProps)
